import java.io.*;
import java.text.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class Survey extends HttpServlet {
    private Connection connection;
    private PreparedStatement updateVotes, totalVotes, results;
    private String error = null;
    // set up database connection and prepare SQL statements
    public void init( ServletConfig config )
	throws ServletException
    {
	// attempt database connection and create PreparedStatements
	try {
	    Class.forName("org.postgresql.Driver");
	    connection = DriverManager.getConnection(
						     "jdbc:postgresql://localhost:5432/animalsurvey", "esempio_user", "esempio_user_password");
	    // PreparedStatement to add one to vote total for a
	    // specific animal
	    updateVotes =
		connection.prepareStatement(
					    "UPDATE surveyresults SET votes = votes + 1 " +
					    "WHERE id = ?");
	    // PreparedStatement to sum the votes
	    totalVotes =
		connection.prepareStatement(
					    "SELECT sum( votes ) FROM surveyresults"
					    );
	    // PreparedStatement to obtain surveyoption table's data
	    results =
		connection.prepareStatement(
					    "SELECT surveyoption, votes, id " +
					    "FROM surveyresults ORDER BY id"
					    );
	}
	// for any exception throw an UnavailableException to
	// indicate that the servlet is not currently available
	catch ( SQLException e ) {
	    error = "SQL problem" + e.getMessage();
	    // throw new UnavailableException( "SQL problem" + e.getMessage() );
	}
	// detect problems loading database driver
	catch ( ClassNotFoundException e ) {
	    error = "Unable to load driver" + e.getMessage();
	    // throw new UnavailableException( "Unable to load driver" + e.getMessage() );
	}
	catch ( Exception exception ) {
	    error = "Generic error " + exception.getMessage();
	    // exception.printStackTrace();
	    // throw new UnavailableException( "Generic error " + exception.getMessage() );
	}
    }
    // end of init method
    // process survey response
    protected void doPost( HttpServletRequest request,
			   HttpServletResponse response )
	throws ServletException, IOException
    {
	// set up response to client
	response.setContentType( "text/html" );
	PrintWriter out = response.getWriter();
	if (error != null) {
	    out.println("<p><b>" + error + "</b></p>");
	    return;
	}
	DecimalFormat twoDigits = new DecimalFormat( "0.00" );
	// start XHTML document
	out.println(
		    "<html><head>");
	// read current survey response
	int value =
	    Integer.parseInt( request.getParameter( "animal" ) );
	// attempt to process a vote and display current results
	try {
	    // update total for current survey response
	    updateVotes.setInt( 1, value );
	    updateVotes.executeUpdate();
	    // get total of all survey responses
	    ResultSet totalRS = totalVotes.executeQuery();
	    totalRS.next();
	    int total = totalRS.getInt( 1 );
	    // get results
	    ResultSet resultsRS = results.executeQuery();
	    out.println( "<title>Thank you!</title></head><body>");
	    out.println( "<p>Thank you for participating." );
	    out.println( "<br />Results:</p>" );
	    // process results
	    int votes;
	    while ( resultsRS.next() ) {
		out.print( resultsRS.getString( 1 )+": " );
		votes = resultsRS.getInt( 2 );
		out.print( twoDigits.format(( double ) votes / total * 100 ) );
		out.print( "% responses: "+ votes );
	    }
	    resultsRS.close();
	    out.print( "Total responses: " + total + "</body></html>");
	    out.close();
	}
	// if database exception occurs, return error page
	catch ( SQLException sqlException ) {
	    // sqlException.printStackTrace();
	    out.println( "<title>Error</title></head>" );
	    out.println( "<body><p>Database error occurred. " );
	    out.println( "Try again later.</p></body></html>" );
	    out.close();
	}
    } // end of doPost method
    // close SQL statements and database when servlet terminates
    public void destroy()
    {
	// attempt to close statements and database connection
	try {
	    updateVotes.close();
	    totalVotes.close();
	    results.close();
	    connection.close();
	}
	// handle database exceptions by returning error to client
	catch( SQLException sqlException ) {
	    // sqlException.printStackTrace();
	}
    } // end of destroy method
}
